///////////////////////////////////////////////////////////////////////
//                                                   41 Post                                       //
// Created by DimasTheDriver in 23/Feb/2010                                      //
// Part of 'Using C# Delegates in Unity3D scripts' post.                      //
// Availiable at:      http://www.41post.com/?p=3146                             //
/////////////////////////////////////////////////////////////////////

using UnityEngine;
using System.Collections;

public class AllEnemies : MonoBehaviour 
{
	//a variable to store the player's character Transform
	private Transform playerTransform;
	//a static boolean variable  to tell if the player has collided with any enemy trigger
	public static bool hasCollided = false;
	
	//an array of game objects, to store every single enemy
	private GameObject[] allEnemies;
	
	/*sets a delegate called 'AllEnemyActions', that returns void and takes a 
	Transform as a parameter*/
	private delegate void AllEnemyActions(Transform pTransform);
	
	/*now that the 'AllEnemyActions' delegate signature is set, we
	instantiate the delegate, naming it as 'aaaDelegate'*/
	private AllEnemyActions aaaDelegate;
	
	void Awake()
	{
		//get the store the player's character Transform
		playerTransform = GameObject.FindWithTag("Player").GetComponent<Transform>();
		
		//get all enemies that are in this scene, and have the 'Enemy' tag
		allEnemies = GameObject.FindGameObjectsWithTag("Enemy");
		
		/* here, the delegate is instantiated. It takes a method as a parameter, meaning that
		*  the FollowPlayer method from the first enemy of the array 'allEnemies' is now wrapped
		*  being wrapped by the aaaDelegate. So if we call the delegate right now, by writing:
		*  aaaDelegate(playerTransform); 
		*  it would be the same as writing:
		*  allEnemies[0].GetComponent<EnemyActions>().FollowPlayer();
		*/
		aaaDelegate = new AllEnemyActions(allEnemies[0].GetComponent<EnemyActions>().FollowPlayer);
		/* now, we add other methods that have the same signature, so the delegate can 
		  * reference them, when it's called.*/
		aaaDelegate += allEnemies[1].GetComponent<EnemyActions>().FollowPlayer;
		aaaDelegate += allEnemies[2].GetComponent<EnemyActions>().FollowPlayer;
		aaaDelegate += allEnemies[3].GetComponent<EnemyActions>().FollowPlayer;
		aaaDelegate += allEnemies[4].GetComponent<EnemyActions>().FollowPlayer;
		//instead of following, the last instantiated enemy should fly,
		aaaDelegate += allEnemies[5].GetComponent<EnemyActions>().Fly;
		
		/* NOTE: It is possible to use a loop to add these methods as references to 
		* the delegate. To remove a reference to a method, you will need to use the '-=' operator*/
	}
	
	void Update()
	{
		//checks if the player has collided with any trigger
		if(hasCollided)
		{
			//call the delegate
			aaaDelegate(playerTransform);
			
			/*The above line is the same as calling every FollowPlayer() method
			* from every GameObject has the 'Enemy' tag. */
		}

	}
}
